<?php
/*
Plugin Name: Plugin Information
Plugin URI: https://blog.tremlas.com/wordpress-plugins/plugin-information-plugin-details/
Description: Adds a shortcode that inserts the equivalent of the plugin information popup available to admins
Version: 3.157.0
Author: David Vines
Author URI: http://blog.tremlas.com
License:  GPLv2 or later
*/
/* © Copyright 2021 David Vines

Plugin Information is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.

Plugin Information is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with {Plugin Name}. If not, see {URI to Plugin License}.*/

include_once 'common.php';

/**
 * Shortcode to add the version of the plugin
 */
function djv_pluginInformation_pluginInformationVersion_shortcode() {
	return 'Plugin Information Version 3.157.0';
}
add_shortcode( 'pluginInformationVersion', 'djv_pluginInformation_pluginInformationVersion_shortcode' );

/**
 * Inserts the equivalent of the plugin information pages for a plugin. The only supported attribute is 'slug'
 */

// Now for a shortcode that can be used to get links and/or tables for the characters
function djv_pluginInformation_shortcode( $attr, $content = null ) {
	global $wpdb;

	if ( isset($attr) && isset( $attr['slug'] ) ) {
		$slug = $attr['slug'];
	} else {
		return "No slug attribute provided on the pluginInformation shortcode";
	}

	$attach = ( isset($attr) && isset( $attr['attachment'] ) && filter_var( $attr['attachment'], FILTER_VALIDATE_BOOLEAN ) );
	$homepage = ( isset($attr) && isset( $attr['homepage'] ) && filter_var( $attr['homepage'], FILTER_VALIDATE_BOOLEAN ) );

	// Remove unnecessary paragraph wrapping
	if (! empty( $content ) ) {
		$patterns = array('/^<\/p>/i', '/<p>$/i');
		$content = preg_replace($patterns, '', $content);
	}

	$api = djv_plugin_get_remote_update_info( '', $slug );

	if ( is_wp_error( $api ) ) {
		return 'Error retrieving the information for the plugin with slug "' . $slug . '"';
	}

	if ( empty($api->sections) ) {
		return "<p>No information successfully retrieved using <q>" . $slug . "</q></p><p>We got </p><pre>" . print_r( $api, true ) . "</pre>";
	}

	$plugins_allowedtags = array(
			'a'          => array(
					'href'   => array(),
					'title'  => array(),
					'target' => array(),
			),
			'abbr'       => array( 'title' => array() ),
			'acronym'    => array( 'title' => array() ),
			'code'       => array(),
			'pre'        => array(),
			'em'         => array(),
			'strong'     => array(),
			'div'        => array( 'class' => array() ),
			'span'       => array( 'class' => array() ),
			'p'          => array(),
			'br'         => array(),
			'ul'         => array(),
			'ol'         => array(),
			'li'         => array(),
			'h1'         => array(),
			'h2'         => array(),
			'h3'         => array(),
			'h4'         => array(),
			'h5'         => array(),
			'h6'         => array(),
			'img'        => array(
					'src'   => array(),
					'class' => array(),
					'alt'   => array(),
			),
			'blockquote' => array( 'cite' => true ),
	);

	// Sanitize HTML.
	foreach ( array( 'version', 'author', 'requires', 'tested', 'homepage', 'downloaded', 'slug' ) as $key ) {
		if ( isset( $api->$key ) ) {
			$api->$key = wp_kses( $api->$key, $plugins_allowedtags );
		}
	}

	$text  = '<table class="plugin-info-table"><tbody><tr><td  class="plugin-info-table-description">' . PHP_EOL;
	// Sansitize the description HTML
	if ( ! empty($api->sections) ) {
		$text .= wp_kses( $api->sections[ 'description' ], $plugins_allowedtags ) . PHP_EOL;
	}
	$text .= '</td><td class="plugin-info-table-fyi">' . PHP_EOL;

	$text .= '<ul>' . PHP_EOL;
	if ( ! empty( $api->version ) ) {
		$text .= '<li><strong>Version:</strong> ';
		if ( ! empty( $api->download_link ) ) {
			if ( $attach ) {
				$id = $wpdb->get_var( "SELECT ID FROM $wpdb->posts WHERE guid LIKE '%" . $api->download_link ."'" );
				$text .= '<a href="' . get_attachment_link( $id ) . '">' . $api->version . '</a>' . PHP_EOL;
			} else {
				$text .= '<a href="' . $api->download_link . '">' . $api->version . '</a>' . PHP_EOL;
			}
		} else {
			$text .= $api->version;
		}
		$text .= '</li>' . PHP_EOL;
	}
	if ( ! empty( $api->author ) ) {
		$text .= '<li><strong>Author:</strong> ' . links_add_target( $api->author, '_blank' ) . '</li>' . PHP_EOL;
	}
	if ( ! empty( $api->last_updated ) ) {
		$plugin_url = plugin_dir_url(__FILE__);
		wp_enqueue_script('djv_pluginInformation_timediff', $plugin_url.'js/timediff.js', array(), '3.157.0' );
		$text .= "<li><strong>Last Updated:</strong> <span data-isotime='" . $api->last_updated . "'>" . $api->last_updated . "</span></li>" . PHP_EOL;
	}
	if ( ! empty( $api->requires ) ) {
		$text .= '<li><strong>Requires WordPress Version:</strong> ' . $api->requires . ' or higher</li>' . PHP_EOL;
	}
	if ( ! empty( $api->tested ) ) {
		$text .= '<li><strong>Compatible up to:</strong> ' . $api->tested . '</li>' . PHP_EOL;
	}
	if ( ! empty( $api->requires_php ) ) {
		$text .= '<li><strong>Requires PHP Version:</strong> ' . $api->requires_php . ' or higher</li>' . PHP_EOL;
	}
	if ( $homepage && ! empty( $api->homepage ) ) {
		$text .= '<li><strong>Plugin homepage:</strong> <a target="_blank" href="' . esc_url( $api->homepage ) . '">click here &#187;</a></li>' . PHP_EOL;
	}
	if ( ! empty( $content ) ) {
		$text .= '<li>' . $content  . '</li>' . PHP_EOL;
	}
	$text .= '</ul></td></tr></table>' . PHP_EOL;

	return $text;
}

add_shortcode( 'pluginInformation', 'djv_pluginInformation_shortcode' );

function djv_pluginInformation_changeLog_shortcode( $attr, $content = null ) {
	global $wpdb;
	if ( isset($attr) && isset( $attr['slug'] ) ) {
		$slug = $attr['slug'];
	} else {
		return "No slug attribute provided on the pluginInformation shortcode";
	}

	$attach = ( isset($attr) && isset( $attr['attachment'] ) && filter_var( $attr['attachment'], FILTER_VALIDATE_BOOLEAN ) );

	$api = djv_plugin_get_remote_update_info( '', $slug );

	if ( is_wp_error( $api ) ) {
		return 'Error retrieving the information for the plugin with slug "' . $slug . '"';
	}

	if ( empty($api->sections) ) {
		return "<p>No information successfully retrieved using <q>" . $slug . "</q></p><p>We got </p><pre>" . print_r( $api, true ) . "</pre>";
	}

	$plugins_allowedtags = array(
			'a'          => array(
					'href'   => array(),
					'title'  => array(),
					'target' => array(),
			),
			'abbr'       => array( 'title' => array() ),
			'acronym'    => array( 'title' => array() ),
			'code'       => array(),
			'pre'        => array(),
			'em'         => array(),
			'strong'     => array(),
			'div'        => array( 'class' => array() ),
			'span'       => array( 'class' => array() ),
			'p'          => array(),
			'br'         => array(),
			'ul'         => array(),
			'ol'         => array(),
			'li'         => array(),
			'h1'         => array(),
			'h2'         => array(),
			'h3'         => array(),
			'h4'         => array(),
			'h5'         => array(),
			'h6'         => array(),
			'img'        => array(
					'src'   => array(),
					'class' => array(),
					'alt'   => array(),
			),
			'blockquote' => array( 'cite' => true ),
	);

	if ( ! empty($api->sections) ) {
		$text = wp_kses( $api->sections[ 'changelog' ], $plugins_allowedtags ) . PHP_EOL;
	}

	if ( $attach ) {
		$dom = new DOMDocument();
		$dom->loadHTML( '<div>' . $text . '</div>' );
		$links = $dom->getElementsByTagName( 'a' );
		foreach($links as $link) {
			$url = $link->getAttribute( 'href' );
			$id = $wpdb->get_var( "SELECT ID FROM $wpdb->posts WHERE guid LIKE '%" . parse_url( $url )['path'] . "'" );
			$link->setAttribute( 'href', get_attachment_link( $id ) );
		}
		$text = substr($dom->saveXML($dom->getElementsByTagName('div')->item(0)), 5, -6);
	}

	return $text;
}
add_shortcode( 'pluginInformationChangeLog', 'djv_pluginInformation_changeLog_shortcode' );


/* --------------------- Functions for updating plugins ------------------------------ */
function djv_pluginInformation_pre_set_site_transient_update_plugins( $transient ) {
	$id = 'pluginInformation/pluginInformation.php';
	$name = 'Plugin Information';
	$slug = 'pluginInformation';
	return djv_plugin_pre_set_site_transient( $transient, $id, $name, $slug );
}
add_filter( 'pre_set_site_transient_update_plugins', 'djv_pluginInformation_pre_set_site_transient_update_plugins' );

function djv_pluginInformation_plugin_info( $result, $action, $args ) {
 $name = 'Plugin Information';
 $slug = 'pluginInformation';
 if ( 'plugin_information' !== $action ) {
  return $result;
 }
 if ( $slug !== $args->slug) {
  return $result;
 }
 return djv_plugin_get_remote_update_info( $name, $slug );
}
add_filter( 'plugins_api', 'djv_pluginInformation_plugin_info', 10, 3 );
?>